/* Jeff Connelly
 * C6 L1 - IRS, taxpayer class
 * 27Sep 02
 */

#include "apstring.h"
#include "irs.h"
#include <stdlib.h>

double TaxPayer::GetFedTax()
{
	double taxed;

	if (m_status == SINGLE)
	{
		if (m_income >= 0 && m_income < 24650)
			taxed = 0.15 * m_income;
		else if (m_income >= 24650 && m_income < 59750)
			taxed = (m_income - 24650) * 0.28 + 3697.50;
		else //if (m_income >= 59750)
			taxed = (m_income - 59750) * 0.31 + 13525.50;
	} else if (m_status == MARRIED) {
		if (m_income >= 0 && m_income < 41200)
			taxed = m_income * 0.15;
		else if (m_income >= 41200 && m_income < 99600)
			taxed = (m_income - 41200) * 0.28 + 6180.00;
		else //if (m_income >= 99600)
			taxed = (m_income - 99600) * 0.31 + 22532.00;
	} else {
		throw IRSTaxingException("undefined filing status");
	}

	return taxed;
}

void TaxPayer::SetIncome(double i)
{
	m_income = i;
}

bool TaxPayer::ValidStatus()
{
	return m_status == MARRIED || m_status == SINGLE;
}

void TaxPayer::SetStatus(FILING_STATUS status)
{
	m_status = status;
}

apstring TaxPayer::GetStatus()
{
	if (m_status == UNDEFINED)
		return "undefined";
	else if (m_status == SINGLE)
		return "single";
	return "married";
}

double TaxPayer::GetIncome()
{
	return m_income;
}

void TaxPayer::Undefine()
{
	m_status = UNDEFINED;
	m_income = 0;
}

TaxPayer::TaxPayer()
{
	Undefine();
}

TaxPayer::TaxPayer(FILING_STATUS status, double income)
{
	m_status = status;
	m_income = income;
}

ostream& operator<<(ostream& os, TaxPayer& citizen)
{
	cout << "Filing status : " << citizen.GetStatus() << endl;
	cout << "Taxable income: $" << citizen.GetIncome() << endl;
	cout << "Federal tax   : $" << citizen.GetFedTax()  << endl;
	return os;
}

istream& operator>>(istream& is, TaxPayer& citizen)
{
	apstring strStatus, strIncome;
	double income;

	while(! citizen.ValidStatus())
	{
		cout << "Enter filing status, m for married s for single (q=quit): ";
		getline(cin, strStatus);

		if (strStatus.length() == 0)
			continue;

		if (strStatus == "throw")
			throw IRSTaxingException("user exception thrown");
		if (strStatus == "st") 
		{
			cout << "Performing self test...";
			citizen.SetStatus(SINGLE);
			citizen.SetIncome(35125);
			if (citizen.GetFedTax() == 6630.5)
			{
				cout << "PASSED\n";
			} else {
				cout << "FAILED\nDiagnostics information:" << endl;
				cout << "Status=" << citizen.GetStatus() << endl;
				cout << "Income=" << citizen.GetIncome() << endl;
				cout << "Calculated federal tax=" << citizen.GetFedTax() << endl;
			}
		}

		if (strStatus[0] == 'm')
			citizen.SetStatus(MARRIED);
		else if (strStatus[0] == 's')
			citizen.SetStatus(SINGLE);
		else if (strStatus[0] == 'q')
			throw IRSTaxingException(1);
	}

	cout << "Enter taxable income: ";
	getline(cin, strIncome);

	income = atof(strIncome.c_str());

	citizen.SetIncome(income);
		
	return is;
}

IRSTaxingException::IRSTaxingException(apstring reason)
{
	m_reason = reason;
	m_quit = 0;
}

apstring IRSTaxingException::Reason()
{
	return m_reason;
}

IRSTaxingException::IRSTaxingException(int quit)
{
	m_reason = "user-initiated abort";
	m_quit = quit;
}

int IRSTaxingException::Quit()
{
	return m_quit;
}