/* Jeff Connelly
 * Initialization and output of apvectors
 * Hereby released into the public domain, may be freely distributed
 * Copyleft (C) 2002 Jeff Connelly. All Rights Reversed.
 * 24 Sep 2002
 */

#include <iostream.h>        // for ostream
#include <stdarg.h>          // for va_*
#include <apvector.h>        // for class apvector<>
#include <apstring.h>        // for class apstring

int init_vector(apvector<int>& v, int len, ...);
apvector<int> make_vector(int len, ...);

// Setup initialization of make_vector_<vt>, where vt is the apvector templated type
// Example:
//
//    SETUP_INITVECT(int);    // Must be in file scope, NOT IN A FUNCTION
// 
// Now make_vector_int can be used, anywhere:
//
//                                //  length  init-elements listed in reverse
//    apvector<int> vect = apvector_int(3, 2, 1, 0);
//
//    vect now stores { 0, 1, 2 };
//
#define SETUP_INITVECT(vt) SETUP_INITVECT_MANUAL(vt, vt, vt);

// SETUP_INITVECT_MANUAL is only needed when vt isn't an identifier, and the second
// parameter fname lets one set the function name
#define SETUP_INITVECT_MANUAL(vt, fname, elt) apvector<vt> apvector_##fname(int len, ...) \
{ \
	apvector<vt> v; \
	int pos = 0; \
	vt i; \
	va_list marker; \
	\
	v.resize(len); \
	\
	va_start(marker, len); \
	while(--len != -1) \
	{ \
		i = va_arg(marker, elt); \
		v[pos++] = i; \
	} \
	va_end(marker); \
	return v; \
} 

// Setup an apvector for each element to be delegated to the stream insertion operator
// Calls ostream& operator<<(ostream& os, apvector<vt>& s) on every element in s
//
// Example:
//      
//    SETUP_OUTVECT(int, "\t");
//
//    apvector<int> v = apvector_int(5, 1, 2, 3, 4, 5);
//
//    cout << v << endl; 
//
// Output:
//
//    1<tab>2<tab>3<tab>4<tab>5<tab><endl>
//

#define SETUP_OUTVECT(vt, sep) ostream& operator<<(ostream& os, const apvector<vt>& s) \
{ \
	char choice = 'a'; \
	for (int i = 0; i < s.length(); i++) \
	{ \
		os << "\t\t" << choice++ << ". " << s[i] << "\n"; \
	} \
	return os; \
}
