/* Jeff Connelly
 * C7 L1
 * Payments
 */

#include <apstring.h>
#include <apvector.h>
#include <conio.h>

double prompt_and_input_double(apstring p, double ndefault);
apstring repeat(int times, char ch);
void browse(double principle, double monthly_rate, double change, int months);

// Screen rows
#define SCROWS 25

// 40 months, total interest 2222.73
// One statement each month
typedef struct tagSTATEMENT
{
	int month;
	double principle;
	double interest;
	double change; // - payment, + investment
	double balance;
} MONTHLY_STATEMENT;

int main()
{
	apstring again;
	do
	{
		double principle, monthly_rate, change;
		int months;
		
		// Obtain inputs
		system("cls");
		cout << "Fill in the following fields, or just press enter to get the value in brackets\n"
		<< endl;

		principle = prompt_and_input_double("Principle", 10000.);
		monthly_rate = prompt_and_input_double("Annual interest rate%", 12.) / 1200;
		change = prompt_and_input_double("Monthly change, payment(-) or investment(+)", -300.);
		if (change > 0)
			months = (int)prompt_and_input_double("Months to iterate", 10);
		else
			months = -1;  // iterate till 0
 
		// Act on inputs
		browse(principle, monthly_rate, change, months);

		cout << endl << endl << "Run again(y): ";
		cin >> again;

		// Y=59=1011001=89
		// y=79=1111001=121
		//      1011111=5F=95 mask out case bit
		// This allows for case-insensitive 
		cin.ignore(80, '\n');
	} while ((again[0] & 95) == 'Y');
	return 0;
}

// Calculate monthly statements and browse them interactively, using the given inputs
void browse(double principle, double monthly_rate, double change, int months = -1)
{
	apvector<MONTHLY_STATEMENT> statements;
	int current_month;
	double interest, total_interest;

	// Set text mode 80x50
	system("mode con lines=25");	

	current_month = 0;
	do
	{
		statements.resize(statements.length() + 1);
		statements[statements.length() - 1].principle = principle;
		
		++current_month;
		interest = principle * monthly_rate;
		principle = principle + interest + change; // New balance
		
		statements[statements.length() - 1].month     = current_month;
		
		statements[statements.length() - 1].interest  = interest;
		statements[statements.length() - 1].change    = change;
		statements[statements.length() - 1].balance   = principle;

		total_interest += interest;

	} while(
		(months == -1) ? (principle + interest + change > 0) // if paid off by next month, finish now
		: --months);   // or if inputed # months, show that many
	
	int lines = 0;

	int i = 0;
	for(;;)    // loop has init/test/change in loop for increased flexibility
	{
#define to(x) x##t##o/*nothing to see here*/
		if (i > statements.length() - 1)    // if last statement, give user control
		{
			i = statements.length() - 1;
			to(go) CONTROL;
		}

		if (i < 0)
			i = 0;
		cout << statements[i].month << "\t" <<
			statements[i].principle << "\t\t" <<
			statements[i].interest << "\t\t" <<
			abs((int)statements[i].change) << "\t" <<
			statements[i].balance << "\t\t\t" << endl << flush;
		lines++; 
		// If either on a screen 
		if (lines == SCROWS - 1 || i == statements.length() - 1)
		{ CONTROL:
			int cmd;
			apstring prompt;

			if (statements[i].change < 0) // negative, payment
				prompt = "Month   Principle       Interest     Payment    Balance (F1=help): ";
			else // positive, investment
				prompt = "Month   Principle       Interest     Investment Balance (F1=help): ";
				//"(more) use up/pgup/dn/pgdn/end/home/esc/insert keys: ";

			cout << prompt << flush;
			//getline(cin, command);
			while (!_kbhit())
				;
			cmd = _getch();
			
			// Backspace over the prompt, clearing it
			cout << repeat(prompt.length() + 1, 8) << flush;

			if (cmd == 224)  // extended keycode requires 2nd getch
				cmd = 224 + _getch();
			if (cmd == 0)    // other form of an extended keycode, 256 + 59=F1
				cmd = 256 + _getch();
			
			if (cmd == 224 + 72) { // (up arrow) previous line
				system("cls"); // CLS is an easy way to go to (0,0), but not good
				lines = 0;
				i -= SCROWS - 1;
				continue;
			} else if (cmd == 224 + 73) { // (pg up) previous page
				lines = 0;
				i -= SCROWS;
				continue;
			} else if (cmd == 224 + 80 || cmd == 13) { // (down arrow/enter) next line
				lines--;
				i++;
				continue;
			} else if (cmd == 224 + 81 || cmd == 32) { // (pg down/space) next page
				lines = 0;
				continue;
			} else if (cmd == 224 + 79) { // (end) last page
				lines = 0;
			} else if (cmd == 224 + 71) { // (home) first page
				lines = 0;
				i = 0;
				continue;
			} else if (cmd == 27) { // (escape) quit
				_getch();
				break;
			} else if (cmd == 256 + 59) { // (F1) help
				system("cls");
				cout << "-- Interest Help System ---" << endl;
				cout << "Up             Previous line\n" 
					    "PgUp           Previous page\n"
						"Down/Enter     Next line\n"
						"PgDn/Spacebar  Next page\n"
						"End            Last page\n"
						"Home           First page\n"
						"Insert         Take out a loan\n"
						"Delete         Sell your stock" << endl;
				system("pause");
				system("cls");
				continue;
			} else if (cmd == 224 + 82) { // (insert) take out a loan
				system("start http://www.bankofamerica.com/loansandhomes/index.cfm/");
			} else if (cmd == 224 + 83) { // (delete)
				system("start http://finance.yahoo.com/");
			} else if (cmd == 's') { // easter egg
				system("start /m sol");
			} else if (cmd == 'm') {
				system("start /m winmine");
			} else if (cmd == 'f') {
				system("start /m freecell");
			}
		}
		i++;
	}
	cout << endl << "Total interest: " << total_interest << endl;
}

double prompt_and_input_double(apstring p, double ndefault)
{
	apstring input_buffer;
	double no;

	cout << p << " [" << ndefault << "]: ";
	getline(cin, input_buffer);
	no = atof(input_buffer.c_str());
	if (!no)
		no = ndefault;
	return no;
}

// Repeats CH, TIMES number of times
// Equivalent to the Perl code ch x times
apstring repeat(int times, char ch)
{
	apstring str;

	while(--times)
		str += ch;
	return str;
}
