/* Jeff Connelly
 * 05 Dec 2002
 * Line Segment (Coordinate) Program
 */

#include <iostream.h>     // for [io]stream
#include <stdlib.h>       // for system
#include <float.h>        // for _finite
#include <math.h>         // for sqrt

// Cartesian point on a plane
struct POINT
{
	double x, y;
};

// Line segment defined by two finite points
struct LINE
{
	POINT p1, p2;
};

istream& operator>>(istream&, POINT&);
istream& operator>>(istream&, LINE&);
ostream& operator<<(ostream& os, POINT& pt);
ostream& operator<<(ostream& os, LINE& L);
double slope(LINE);
double xintercept(double m, double x, double y);
double length(LINE line);
POINT midpoint(LINE line);
int operator==(LINE& lhs, LINE& rhs);
int operator==(POINT& lhs, POINT& rhs);

int main()
{
	LINE L;
	double m;


	cout << "Line Slope Program. To *not* run again, hit Control-C" << endl;

	for(;;)
	{
		// try (7,5) - (9,2)
		cin >> L;

		cout << endl << 
			    "Information for line:        " << L << endl;
		
		if (L.p1 == L.p2)
			cout << "(warning: non-unique ordered pairs)" << endl;

		cout << endl << 
			    "The slope of the line is:    " << flush;
		m = slope(L);
		if (!_finite(m))
			cout << "undefined (vertical line x=" << (L.p1.x) << ")";
		else
			cout << m;
		cout << endl;

		
		//cout << endl << "X-Intercept: " << xintercept(m, L.p1.x, L.p1.y) << endl;
		cout << "Length of the line segment:  " << length(L) << endl;
		cout << "The midpoint of the line is: " << midpoint(L) << endl;
		cout << endl << endl;
	}
	
	return 0;
}

POINT midpoint(LINE line)
{
	POINT pt;
	pt.x = (line.p1.x + line.p2.x) / 2;
	pt.y = (line.p1.y + line.p2.y) / 2;
	return pt;
}

double length(LINE line)
{
	return sqrt((line.p2.x - line.p1.x) * (line.p2.x - line.p1.x) +
		(line.p2.y - line.p1.y) * (line.p2.y - line.p1.y));
}

// (x1,y1)
// (x2,y2)
// y = mx + b
// y = ((x1-x2)/(y1-y2))x + b
// y = -1/2x + b
// y1 = -1/2(x1) + b
//      5 = -1/2(7) + b
//--1/2(7)= -1/2(7)
// b = y1 - mx
double xintercept(double m, double x, double y)
{
	return y - m * x;
}

double slope(LINE line)
{
	return (line.p2.y - line.p1.y) / (line.p2.x - line.p1.x);
}

// cin >> point
istream& operator>>(istream& is, POINT& pt)
{
	cout << "Enter the x coordinate: ";
	is >> pt.x;
	cout << "Enter the y coordinate: ";
	is >> pt.y;
	return is;
}	

// cin >> line
istream& operator>>(istream& is, LINE& L)
{
	cout << "-- first point --" << endl;
	is >> L.p1;
	cout << "-- second point --" << endl;
	is >> L.p2;
	
	return is;
}

// cout << point
ostream& operator<<(ostream& os, POINT& pt)
{
	os << "(" << pt.x << ", " << pt.y << ")";
	return os;
}

// cout << line
ostream& operator<<(ostream& os, LINE& L)
{
	os << L.p1 << "-" << L.p2;
	return os;
}

int operator==(POINT& lhs, POINT& rhs)
{
	return (lhs.x == rhs.x && lhs.y == rhs.y);
}

int operator==(LINE& lhs, LINE& rhs)
{
	return (lhs.p1 == rhs.p1 && lhs.p2 == rhs.p2);
}